
/// helper to print values
function print(val){
    let el = document.createElement('p')
    el.innerText = val
    document.body.appendChild(el)
}


/// 1) Stream - Aggiungere elementi ad HTML

const observable = Rx.Observable.create( observer => {
    observer.next( 'hello' )
    observer.next( 'world' )
})

observable.subscribe(val => print(val))


/// 2) DOM Events - Cattura del movimento del mouse
const clicks = Rx.Observable.fromEvent(document, 'click')

clicks.subscribe(click => console.log(click))
// click around the web page...
// MouseEvent<data>
// MouseEvent<data>

/// 3) Observable from Promise - Aggiunta temporizzata di un elemento
const promise = new Promise((resolve, reject) => { 
    setTimeout(() => {
        resolve('resolved!')
    }, 7000)
});

const obsvPromise = Rx.Observable.fromPromise(promise)

obsvPromise.subscribe(result => print(result) ) 

// wail 7 second...
// resolved!

/// 4) Observable Time Interval - Creazione di elementi in modo lazy
const interval = Rx.Observable.interval(1000)

interval.subscribe(i => print( i ))
// 0
// 1
// every second for eternity...


/// 5)  Funzione Map - Caricamento da un JSON senza caricarlo interamente
const jsonString = '{ "Tipo":"Materia", "Nome": "Paradigmi e linguaggi di programmazione"}'
const apiCall = Rx.Observable.of(jsonString);

apiCall
    .map(json => JSON.parse(json))
    .subscribe(obj => {
        print(obj.Tipo)
        print(obj.Nome)
    })

// emit as JS object, rather than useless JSON string

/// 6) Funzione Filter - non carica tutti i valori
const numbers = Rx.Observable.of(1,2,-5,4,6)

numbers
.filter(number => number >= 2)
.subscribe(number => print(number))

/// 7) Throttle - Gestire Stream Overload
const mouseEvents = Rx.Observable.fromEvent(document, 'mousemove')

mouseEvents
   // .throttleTime(1000) // Prova a decommentare 
    .subscribe( e => print(e.type))
// MouseEvent<data>
// wait 1 second...

